from __future__ import absolute_import
import logging
from rest_framework import serializers
from django.conf import settings
from .api import validation, IntentsSchema

log = logging.getLogger(__name__)


class Obj(object):
    def __init__(self, data):
        self.__dict__.update(data)


class BaseASKSerializer(serializers.Serializer):
    
    def create(self, validated_data):
        return Obj(data=validated_data)


class ASKApplicationSerializer(BaseASKSerializer):
    applicationId = serializers.CharField(validators=[validation.validate_app_ids])


class ASKUserSerializer(BaseASKSerializer):
    userId = serializers.CharField()


class ASKSessionSerializer(BaseASKSerializer):
    sessionId = serializers.CharField()
    application = ASKApplicationSerializer()
    attributes = serializers.DictField(required=False)
    user = ASKUserSerializer()
    new = serializers.BooleanField()


class ASKIntentSerializer(BaseASKSerializer):
    name = serializers.CharField()
    slots = serializers.DictField()


class ASKRequestSerializer(BaseASKSerializer):
    type = serializers.CharField()
    requestId = serializers.CharField()
    timestamp = serializers.DateTimeField(format="%Y-%m-%dT%H:%M:%S")
    intent = ASKIntentSerializer(required=False)
    reason = serializers.CharField(required=False)


class ASKOutputSpeechSerializer(BaseASKSerializer):
    type = serializers.ChoiceField(choices=("PlainText", "SSML"))
    text = serializers.CharField() # TODO Choice validation to check if text or ssml is filed
    ssml = serializers.CharField()


class ASKCardSerializer(BaseASKSerializer):
    type = serializers.ChoiceField(choices=("Simple", "LinkAccount"))
    title = serializers.CharField(required=False)
    content = serializers.CharField(required=False)


class ASKRempromptSerializer(BaseASKSerializer):
    outputSpeech = ASKOutputSpeechSerializer(required=False)


class ASKResponseSerializer(BaseASKSerializer):
    outputSpeech = ASKOutputSpeechSerializer(required=False, validators=[validation.validate_char_limit])
    card = ASKCardSerializer(required=False, validators=[validation.validate_char_limit])
    reprompt = ASKRempromptSerializer(required=False)
    shouldEndSession = serializers.BooleanField()


class ASKSerializer(BaseASKSerializer):
    version = serializers.FloatField(required=True)
    
    session = ASKSessionSerializer(write_only=True)
    request = ASKRequestSerializer(write_only=True)
    
    sessionAttributes = serializers.DictField(required=False, read_only=True)
    response = ASKResponseSerializer(read_only=True)
    
    def create(self, validated_data):
        intent_name = validated_data["request"]["intent"]["name"]
        intent_kwargs = {}
        for slot, slot_data in validated_data["request"]["intent"]["slots"].items():
            intent_kwargs[slot_data["name"]] = slot_data['value']
        log.info("Routing: {0} - {1}".format(intent, intent_kwargs))
        response = IntentsSchema.route(intent_name, intent_kwargs)
        if isinstance(response, ASKResponseSerializer) is not True:
            msg = "Intent '{0}' does not return an ASKResponseSerializer"
            raise serializers.ValidationError(detail=msg.format(intent_name))
        validated_data['response'] = response
        # TODO: handle session attributes somehow
        return Obj(data=validated_data)





from __future__ import absolute_import
from ..serializers import ASKResponseSerializer, ASKOutputSpeechSerializer, ASKRempromptSerializer, ASKCardSerializer


class ResponseBuilder(object):
    """
    Simple class to help users to build alexa responses
    """

    @classmethod
    def create_response(cls,
                        message=None, message_is_ssml=False,
                        reprompt=None, reprompt_is_ssml=False,
                        title=None, content=None, card_type=None,
                        end_session=True):
        """
        Shortcut to create a fully baked ASKResponseSerializer
        
        Output Speech:
        message - text message to be spoken out by the Echo
        message_is_ssml - If true the "message" is ssml formated and should be treated as such
        
        Reprompt Speech:
        reprompt - text message to be spoken out by the Echo
        reprompt_is_ssml - If true the "repropt" is ssml formated and should be treated as such
        
        Card:
        card_type - A string describing the type of card to render.
        title - A string containing the title of the card. (not applicable for cards of type LinkAccount).
        content - A string containing the contents of the card (not applicable for cards of type LinkAccount).
                  Note that you can include line breaks in the content for a card of type Simple.
        
        end_session - flag to determine whether this interaction should end the session
        
        For more comprehensive documentation see:
        https://developer.amazon.com/public/solutions/alexa/alexa-skills-kit/docs/alexa-skills-kit-interface-reference
        """
        data = {}
        data['shouldEndSession'] = end_session
        if message:
            data['outputSpeech'] = cls.create_speech(message=message,
                                                     is_ssml=message_is_ssml)
        if title or content:
            data['card'] = cls.create_card(title=title,
                                           content=content,
                                           card_type=card_type)
        if reprompt:
            data['reprompt'] = cls.create_reprompt(message=reprompt,
                                                   is_ssml=reprompt_is_ssml)
        response = ASKResponseSerializer(data=data)
        response.is_valid(raise_exception=True)
        return response

    @classmethod
    def create_speech(cls, message=None, is_ssml=False):
        data = {}
        if is_ssml:
            data['type'] = "SSML"
            data['ssml'] = message
        else:
            data['type'] = "PlainText"
            data['text'] = message
        speech = ASKOutputSpeechSerializer(data=data)
        speech.is_valid(raise_exception=True)
        return speech

    @classmethod
    def create_reprompt(cls, message=None, is_ssml=False):
        data = {}
        data['outputSpeech'] = cls.create_speech(message=message,
                                                 is_ssml=is_ssml)
        reprompt = ASKRempromptSerializer(data=data)
        reprompt.is_valid(raise_exception=True)
        return reprompt
     
    @classmethod
    def create_card(cls, title=None, content=None, card_type=None):
        data = {"type": card_type or "Simple"}
        if title: data["title"] = title
        if content: data["content"] = content
        card = ASKCardSerializer(data=data)
        card.is_valid(raise_exception=True)
        return card


from __future__ import absolute_import
from rest_framework.serializers import Serializer
from . import validation, fields
from .intents_schema import intent, IntentsSchema
from .response_builder import ResponseBuilder




from __future__ import absolute_import
from rest_framework.status import HTTP_200_OK
from rest_framework.viewsets import GenericViewSet
from rest_framework.response import Response
from . import serializers
from .api import validation


class ASKViewSet(GenericViewSet):
    serializer_class = serializers.ASKSerializer
    
    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        response = Response(serializer.data, status=HTTP_200_OK)
        validation.validate_reponse_limit(response)
        return response

    def dispatch(self, request, *args, **kwargs):
        validation.validate_alexa_request(request)
        return super(ASKViewSet, self).dispatch(request, *args, **kwargs)



from __future__ import absolute_import
import json
from django.core.management.base import BaseCommand, CommandError
from ...api import IntentsSchema

class Command(BaseCommand):
    help = 'Prints the Alexa Skills Kit intents schema'

    def handle(self, *args, **options):
        data = IntentsSchema.generate_schema()
        self.stdout.write(json.dumps(data, indent=4, sort_keys=True))


from __future__ import absolute_import
import logging
import warnings
from string import Formatter
from rest_framework import serializers

log = logging.getLogger(__name__)

# This maps DRF serializer fields to the amazon intent slot types
INTENT_SLOT_TYPES = {
    "CharField": "AMAZON.LITERAL",
    "IntegerField": "AMAZON.NUMBER",
    "DateField": "AMAZON.DATE",
    "TimeField": "AMAZON.TIME",
    "DurationField": "AMAZON.DURATION",
}

# These are the valid intent slot types
# We add choicefield here because it represents custom slot types but has to
# have a custom name mapping for the slot type which based on the field label
VALID_SLOT_TYPES = INTENT_SLOT_TYPES.keys() + [
    "ChoiceField"
]

# These utterances can have utterance overrides but need a prefix of "AMAZON."
UTTERANCES_OVERRIDE_INTENTS = [
    'HelpIntent',
    'CancelIntent',
    'StopIntent'
]

class IntentsSchema():
    intents = {}

    @classmethod
    def route(cls, name, data=None):
        """Routes an intent to the proper method"""
        if name not in cls.intents.keys():
            msg = "Unable to find an intent defined for '{0}'"
            raise serializers.ValidationError(detail=msg.format(name))
        kwargs = {}
        func, serializer = cls.intents[name]
        if serializer:
            if data is None:
                msg = "Intent '{0}' requires slots data and none was provided"
                raise serializers.ValidationError(detail=msg.format(name))
            else:
                slots = serializer(data=data)
                slots.is_valid(raise_exception=True)
                kwargs.update(slots.data)
        return func(**kwargs)

    @classmethod
    def register(cls, func, name, serializer=None):
        if serializer:
            s = serializer()
            for field_name, field in s.get_fields().items():
                if field.__class__.__name__ not in VALID_SLOT_TYPES:
                    msg = "'{0}' on serializer '{1}' is not a valid alexa slot type"
                    raise ValueError(msg.format(field_name,
                                                s.__class__.__name__))
        cls.intents[name] = (func, serializer)

    @classmethod
    def generate_schema(cls):
        """Generates the alexa intents schema json"""
        intents =[]
        for intent_name in cls.intents.keys():
            intent_data = {"intent": intent_name,
                           "slots": []}
            _, serializer = cls.intents[intent_name]
            if serializer:
                s = serializer()
                for field_name, field in s.get_fields().items():
                    slot_type = INTENT_SLOT_TYPES.get(field.__class__.__name__,
                                                 field.label)
                    if slot_type is None:
                        msg = "Intent '{0}' slot '{1}' does not have a valid slot_type"
                        raise ValueError(msg.format(intent_name, field_name))
                    if slot_type == "AMAZON.LITERAL":
                        msg = "Please upgrade intent '{0}' slot '{1}' to a ChoiceField with choices!"
                        log.warning(msg.format(intent_name, field_name))
                    slot_data = {
                        "name": field_name,
                        "type": slot_type
                    }
                    intent_data['slots'].append(slot_data)
            intents.append(intent_data)
        return {"intents": intents}

    @classmethod
    def generate_utterances(cls):
        """Generates the alexa utterances schema for all intents"""
        utterance_format = "{0} {1}"
        utterances =[]
        for intent_name in cls.intents.keys():
            func, serializer = cls.intents[intent_name]
            fields = []
            if serializer:
                s = serializer()
                fields = s.get_fields().keys()
            docstring = """"""
            if func.__doc__:
                if "---\n" in func.__doc__:
                    docstring = func.__doc__.split("---")[-1].strip()
            for line in docstring.splitlines():
                line = line.strip()
                for key in [i[1] for i in Formatter().parse(line) if i[1]]:
                    if "|" in key:
                        key = key.split("|")[-1]
                    if key not in fields:
                        msg = "Intent '{0}' utterance '{1}' has a missing the key in the serializer '{2}'"
                        raise ValueError(msg.format(intent_name,
                                                    line,
                                                    s.__class__.__name__))
                if intent_name in UTTERANCES_OVERRIDE_INTENTS:
                    intent_name = "AMAZON.{0}".format(intent_name)
                utterances.append(utterance_format.format(intent_name, line))
        return utterances


def intent(*args, **kwargs):
    """
    Decorator that registers a function to the IntentsSchema
    """
    invoked = bool(not args or kwargs)
    if not invoked:
        func, args = args[0], ()

    def register(func):
        name = kwargs.get('name', func.__name__)
        serializer = kwargs.get('serializer', None)
        IntentsSchema.register(func, name, serializer)
        return func
    return register if invoked else register(func)
